#!/usr/bin/env python
# -*- coding:utf-8 -*-

import os
import sys


## get NEs from IOB formated ones
def getNEs(chunks: list):

    buff = []
    ret = []

    for i, chunk in zip(reversed(range(len(chunks))), reversed(chunks)):

        word, tag = chunk.split("/")

        if tag == "O":
            buff.clear()
        else:
            buff += [word]

            if tag[-1] == "B":
                ret += [(i, " ".join(buff[::-1]), tag.split("-")[0])]
                buff.clear()

    return ret


## calculate precision, recall, and f-measure
def accuracy(
    ref_list: list,
    hyp_list: list,
    quiet: bool=True,
):

    n_matched = 0
    n_ref_total = 0
    n_hyp_total = 0

    for ref, hyp in zip(ref_list, hyp_list):

        ref_nes = getNEs(ref)
        hyp_nes = getNEs(hyp)

        n_ref_total += len(ref_nes)
        n_hyp_total += len(hyp_nes)

        for ne in ref_nes:
            if ne in hyp_nes:
                n_matched += 1

    eps = 1e-9

    precision = n_matched / n_hyp_total
    recall    = n_matched / n_ref_total
    f_measure = 2 * precision * recall / ( precision + recall + eps )

    if not quiet:
        print("# tags in reference : {}".format(n_ref_total))
        print("# tags in hypothesis: {}".format(n_hyp_total))
        print("# true in hypothesis: {} (/ {})".format(n_matched, n_hyp_total))
        print("Prec.    : ", precision)
        print("Recall   : ", recall)
        print("F-measure: ", f_measure)

    return precision, recall, f_measure
                

def main():
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument("--ref", type=str, default=None, help="ref")
    parser.add_argument("--hyp", type=str, default=None, help="hyp")
    args = parser.parse_args()

    ref_list = [ line.rstrip().split(" ") for line in open(args.ref).readlines() ]
    hyp_list = [ line.rstrip().split(" ") for line in open(args.hyp).readlines() ]
    
    accuracy(ref_list, hyp_list, False)


if __name__ == "__main__":
    main()


