#!/usr/bin/env python
# -*- coding:utf-8 -*-

import torch


class NgramFeatures():

    def __init__(
        self,
        dict_file: str,
        n_max: int=4,
    ):
        self.n_max = n_max

        self.ngrams = {
            k: set()
            for k in range(1, n_max + 1)
        }

        for line in open(dict_file).readlines():

            line = line.rstrip().split(" ")
            if len(line) > 0 and len(line) <= n_max:
                self.ngrams[len(line)].add(" ".join(line))

    ## calculate a feature of a sentence
    def calcFeature(
        self,
        words: list,
        n: int,
    ):
        feat = torch.zeros(len(words))
        for i in range(0, len(words)-n-1):

            ngram = " ".join(words[i:i+n])
            if ngram in self.ngrams[n]:
                feat[i:i+n] = 1

        return feat

    ## calculate features of sentences in a mini-batch
    def calcFeaturesBatch(
        self,
        batch_words: list,
    ):
        batch_size = len(batch_words)
        batch_word_lens = [len(x) for x in batch_words]
        max_len = max(batch_word_lens)

        feats = torch.zeros(batch_size, max_len, self.n_max)
        for i in range(batch_size):
            for j in range(self.n_max):
                feats[i, :len(batch_words[i]), j] = self.calcFeature(batch_words[i], j+1)

        return feats


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument("--dict", type=str, default=None, help="dictionary")
    parser.add_argument("--file", type=str, default=None, help="input file")
    args = parser.parse_args()

    feat_handler = NgramFeatures(args.dict, 4)

    lines = [ 
        [ chunk.split("/") for chunk in line.rstrip().split(" ") ]
        for line in open(args.file).readlines()
    ]

    words = [ [c[0] for c in line] for line in lines ]
    tags = [ [c[1] for c in line] for line in lines ]

    for bw, bt in zip(words, tags):

        print(bw)
        feat = feat_handler.calcFeatureBatch([bw])
        print(feat)
        

