#!/usr/bin/env python
# -*- coding:utf-8 -*-

import copy
import numpy as np
from collections import defaultdict

from tokens import *


class Vocabulary:
    def __init__(
        self,
        lines: list=None,
        freq : int=2,
    ):
        self.t2i = {} ## token to index
        self.i2t = {} ## index to token

        if lines:
            self.build(lines, freq)

    def build(
        self,
        lines   : list,
        freq_min: int,
    ):
        
        t2f = defaultdict(int) ## token to frequency

        for line in lines:
            for token in line:
                t2f[token] += 1

        for token in (UNK, BOS, EOS):
            self.t2i[token] = len(self.t2i)

        for token, freq in t2f.items():
            if not token in self.t2i.keys() and freq >= freq_min:
                self.t2i[token] = len(self.t2i)

        self.i2t = {
            val: key
            for key, val in self.t2i.items()
        }

    def tokens2ids(
        self,
        tokens: list,
    ):
        return [
            self.t2i[w]
            if w in self.t2i.keys() else self.t2i[UNK]
            for w in tokens
        ]

    def ids2tokens(
        self,
        ids: list,
    ):
        return [
            self.i2t[i]
            for i in ids
        ]


## create word, char, tag vocabularies
def createVocabularies(
    file_path   : str,
    ne_list_path: str,
    freq_word   : int=2,
    freq_char   : int=2,
):

    ## build word/char vocab
    words_list = []
    chars_list = []

    for line in open(file_path).readlines():

        chunks = line.rstrip().split(" ")
        words = [ c.split("/")[0] for c in chunks ]
        words_list += [words]

        temp = []
        for word in words:
            temp += list(word)
        chars_list += [temp]

    word_vocab = Vocabulary(words_list, freq_word)
    char_vocab = Vocabulary(chars_list, freq_char)

    ## build tag voab
    tag_vocab = Vocabulary()
    tag_vocab.t2i = { START: 0, STOP: 1, "O": 2 }
    for line in open(ne_list_path).readlines():
        
        line = line.rstrip()
        tag_vocab.t2i[line + "-B"] = len(tag_vocab.t2i)
        tag_vocab.t2i[line + "-I"] = len(tag_vocab.t2i)

    tag_vocab.i2t = {
        val: key
        for key, val in tag_vocab.t2i.items()
    }

    return word_vocab, char_vocab, tag_vocab


